% ACCURACY - computes accuracy metrics on a (set of) truth table(s)
% truthtablecells - matrix of n.m 2x2 truthtables (col - actual, T,F, row -
%		modeled T, F) in a cell array. truthtablecells is an output of
%		truthtable.m where rows represent different prediction models,
%		columns one or several thresholds
% result n.m matrix of n accuracy metrics (TP + TN) / (TP + FP + FN + TN)
%
% Truth Table:
%						Actual
%                 Positive Negative
%                    .----|----.
%           Positive | TP | FP |
% Predicted          |----|----|
%           Negative | FN | TN |
%                    .----|----.

function accuracyarray = accuracy(truthtablecells)

n1_tt = size(truthtablecells,1);
n2_tt = size(truthtablecells,2);
matarray = cell2mat(truthtablecells);

% TP : 1,1 : ((1:n1_tt)*2-1), ((1:n2_tt)*2-1)
% FP : 1,2 : ((1:n1_tt)*2-1), ((1:n2_tt)*2)
% FN : 2,1 : ((1:n1_tt)*2),   ((1:n2_tt)*2-1)
% TN : 2,2 : ((1:n1_tt)*2),   ((1:n2_tt)*2)
TP = matarray( ((1:n1_tt)*2-1), ((1:n2_tt)*2-1) );
FP = matarray( ((1:n1_tt)*2-1), ((1:n2_tt)*2)   );
FN = matarray( ((1:n1_tt)*2),   ((1:n2_tt)*2-1) );
TN = matarray( ((1:n1_tt)*2),   ((1:n2_tt)*2)   );

accuracyarray = ( TP + TN ) ./ ( TP + TN + FP + FN );
% accuracyarray = ( matarray([(1:n_tt)*2-1],1)  + matarray([(1:n_tt)*2],2)  ) ./ ...
% 	 (	matarray([(1:n_tt)*2-1],1)  + matarray([(1:n_tt)*2],2) +...
% 		matarray([(1:n_tt)*2-1],2) + matarray([(1:n_tt)*2],1) );