% AUROC - computes area under ROC curve for a binary classfier using an
%	approximation of Wilcoxon's statistic.
%	For details refer to: Fogarty, J., Baker, R. S., and Hudson, S. E. 
%		(2005). Case studies in the use of ROC curve analysis for sensor-
%		based estimates in human computer interaction. In Inkpen, K. and 
%		van de Panne, M., editors, Proceedings of Graphics Interface 
%		Conference (GI 2005), pages 129?136, Victoria, British Columbia, 
%		Canada. Canadian Human-Computer Communications Society.
% data - 1st column actual binary output, 2nd..nth - modeled output.
% aprime - column vector of n-1 AU ROC measures.
function [aprime, aprime_se] = auroc(data)

ncol = size(data,2);
np = sum(data(:,1)==1); % number of actual positive cases
nn = sum(data(:,1)==0); % number of actual negative cases

idx_p = find(data(:,1)==1); % indices of actual positive cases
idx_n = find(data(:,1)==0); % indices of actual negative cases

aprime = zeros(ncol-1,1);

for data_i=2:ncol,
	for i=1:np,
		sp = data(idx_p(i), data_i);
		
% 		%this is an alternative to the cycle on j, sadly - working slower
% 		aprime = aprime + sum(sp>predicted(idx_n)) + sum(sp==predicted(idx_n))/2;

		for j=1:nn,
			sn = data(idx_n(j), data_i);
			aprime(data_i-1) = aprime(data_i-1) + (sp>sn) + (sp==sn)/2;
		end;
	end;
end;
aprime = aprime / (np*nn);

Dp = (np-1) * ( aprime / (2-aprime) - aprime^2 );
Dn = (nn-1) * ( 2*aprime^2 / (1+aprime) - aprime^2);
aprime_se = sqrt( (aprime*(1-aprime) + Dp + Dn)/(np*nn) );
