% CUMULATE X BY ONE ACTION

% INPUT
% source_data - raw data structure from blendmodel_compute
% para - all model parameters
% OUTPUT
% ssr - sum of squared errors
% ssr_percent - % of sum of squared errors from total errors possible
% model - computed model

% WARNING combination of user_param & problem_param cannot exceed 2^WEIGHT_POWER


function model = cumulateL_each(source_data, model_data, log_idx)

% constants
IDX_USER = 1;
IDX_ACTIVITY = 2;
IDX_RESULT = 3;
IDX_DATE = 4;
% --
	
idx_u = source_data.problem_log(log_idx, IDX_USER);
idx_a = source_data.problem_log(log_idx, IDX_ACTIVITY);
res = source_data.problem_log(log_idx, IDX_RESULT);
idx_c = find( source_data.problem_concept(idx_a,:) )';
idx_c_dir = source_data.problem_concept_dir(idx_a,idx_c)';
idx_c_wght = source_data.problem_concept(idx_a,idx_c)';

out_w_sum = sum(idx_c_wght(find(idx_c_dir==1),:));
pre_w_sum = sum(idx_c_wght(find(idx_c_dir==0),:));

model_data.probl_lrnop     (idx_u,idx_a) = model_data.probl_lrnop     (idx_u,idx_a) + 1; % inc problem attempt
model_data.probl_lrnop_corr(idx_u,idx_a) = model_data.probl_lrnop_corr(idx_u,idx_a) + res; % inc problem attempt

model_data.con_lrnop     (idx_u,idx_c) = model_data.con_lrnop     (idx_u,idx_c) + 1;
model_data.con_lrnop_corr(idx_u,idx_c) = model_data.con_lrnop_corr(idx_u,idx_c) + res;

% compute ssr % pv log
% ssr = ssr + (res - model_data.solves(idx_u, idx_a))^2;
model_data.pv_usr_log{idx_u}  = [model_data.pv_usr_log{idx_u};  res, model_data.solves(idx_u, idx_a)];
model_data.pv_prob_log{idx_a} = [model_data.pv_prob_log{idx_a}; res, model_data.solves(idx_u, idx_a)];
% --

% CUMULATE L
w_sum = ones(size(idx_c)) * pre_w_sum;
w_sum = w_sum .* (idx_c_dir~=1);
w_sum = w_sum + (idx_c_dir==1) * out_w_sum;
multiplier = ones(size(idx_c)) - ( model_data.model(idx_u,idx_c)'<.5 )*.5;

weight = power(idx_c_wght,.25) ./ power( ( model_data.probl_lrnop_corr(idx_u,idx_a)+1) * w_sum, .25);
cum_update = res * weight' .* power( 1-model_data.model(idx_u,idx_c),2) .* multiplier';
model_data.model(idx_u,idx_c) = model_data.model(idx_u,idx_c) + cum_update;

% % update volumes 
% model_data.con_updvol(idx_u,idx_c) = model_data.con_updvol(idx_u,idx_c) + cum_update; 
% model_data.probl_updvol(idx_u,idx_a) = model_data.probl_updvol(idx_u,idx_a) + mean(cum_update); 

% -- CUMULATE L


% % CUMULATE X
% w_sum = ones(size(idx_c)) * pre_w_sum;
% w_sum = w_sum .* (idx_c_dir~=1);
% w_sum = w_sum + (idx_c_dir==1) * out_w_sum;
% 
% weight = idx_c_wght ./ w_sum;
% prev_corr_attempts = model_data.probl_lrnop_corr(idx_u,idx_a) - res;
% p_penalty = 1/(prev_corr_attempts^practice_penalty);
% 
% if(isinf(p_penalty)), p_penalty = 1; end;
% 
% % if not previously set, init
% unset_idx = find(model_data.model(idx_u,idx_c)==0);
% if( and( numel(unset_idx)>0, problem_k > 0 ) ),
% 	model_data.model(idx_u,idx_c(unset_idx)) = problem_k;
% end;
% 
% model_data.model(idx_u,idx_c) = model_data.model(idx_u,idx_c) + ...
% 	res * (1-model_data.model(idx_u,idx_c)) .* weight' * problem_param * p_penalty;
% % -- CUMULATE X



model_data.solves(idx_u, idx_a) = prod( model_data.model(idx_u,idx_c) );

model = model_data;