% CUMULATE BY PROBLEM SSR

% INPUT
% source_data - raw data structure from blendmodel_compute
% para - all model parameters
% OUTPUT
% ssr - sum of squared errors
% ssr_percent - % of sum of squared errors from total errors possible
% model - computed model

% WARNING combination of user_param & problem_param cannot exceed 2^WEIGHT_POWER


function [ssr, percent, model] = cumulateX_all(source_data, para)

% constants
WEIGHT_POWER = 4;
IDX_USER = 1;
IDX_ACTIVITY = 2;
IDX_RESULT = 3;
IDX_DATE = 4;
% --

% params
problem_k = para(1);
problem_param = para(2);
practice_penalty = para(3);
problem_id = para(4);
% --

ssr = 0;

n_users = size(source_data.user_id, 1);
n_probl = size(source_data.problem_id, 1);
n_concept = size(source_data.concept_id, 1);

model_data = struct;
model_data.cum =				ones(  n_users, n_concept )*problem_k;
model_data.cum_solves =			zeros( n_users, n_probl   );
model_data.con_lrnop =			zeros( n_users, n_concept ); 
model_data.con_lrnop_corr =		zeros( n_users, n_concept ); 
model_data.probl_lrnop =		zeros( n_users, n_probl   ); 
model_data.probl_lrnop_corr =	zeros( n_users, n_probl   ); 
model_data.pv_usr_log =			cell ( n_users, 1		  );
for i=1:n_users, model_data.pv_usr_log{i} = []; end;

if(problem_id~=0),
	log_idx = find( source_data.problem_log(:,IDX_ACTIVITY)== problem_id );
	log = source_data.problem_log(log_idx,:);
else
	log = source_data.problem_log;%(log_idx,:);
end;

for log_i = 1:size(log,1),
	
	idx_u = log(log_i, IDX_USER);
	idx_a = log(log_i, IDX_ACTIVITY);
	res = log(log_i, IDX_RESULT);
	idx_c = find( source_data.problem_concept(idx_a,:) )';
	idx_c_dir = source_data.problem_concept_dir(idx_a,idx_c)';
	idx_c_wght = source_data.problem_concept(idx_a,idx_c)';

	out_w_sum = sum(idx_c_wght(find(idx_c_dir==1),:));
	pre_w_sum = sum(idx_c_wght(find(idx_c_dir==0),:));

	model_data.probl_lrnop     (idx_u,idx_a) = model_data.probl_lrnop     (idx_u,idx_a) + 1; % inc problem attempt
	model_data.probl_lrnop_corr(idx_u,idx_a) = model_data.probl_lrnop_corr(idx_u,idx_a) + res; % inc problem attempt

	model_data.con_lrnop     (idx_u,idx_c) = model_data.con_lrnop     (idx_u,idx_c) + 1;
	model_data.con_lrnop_corr(idx_u,idx_c) = model_data.con_lrnop_corr(idx_u,idx_c) + res;

	% compute ssr % pv log
	ssr = ssr + (res - model_data.cum_solves(idx_u, idx_a))^2;
	model_data.pv_usr_log{idx_u} = [model_data.pv_usr_log{idx_u}; res, model_data.cum_solves(idx_u, idx_a)];
	% --
	
	w_sum = ones(size(idx_c)) * pre_w_sum;
	w_sum = w_sum .* (idx_c_dir~=1);
	w_sum = w_sum + (idx_c_dir==1) * out_w_sum;
	
	weight = idx_c_wght ./ w_sum;
	prev_corr_attempts = model_data.probl_lrnop_corr(idx_u,idx_a) - res;
	p_penalty = 1/((prev_corr_attempts+2)^practice_penalty);
	if(isinf(p_penalty)),
		p_penalty = 1;
	end;
% 	weight = power( (idx_c_wght * problem_param) ./ ...
% 		( ( model_data.probl_lrnop_corr(idx_u,idx_a)+1) * w_sum ) , 1/WEIGHT_POWER );
	% update model
	model_data.cum(idx_u,idx_c) = model_data.cum(idx_u,idx_c) + ...
		res * (1-model_data.cum(idx_u,idx_c)) .* weight' * problem_param * p_penalty;

	model_data.cum_solves(idx_u, idx_a) = prod( model_data.cum(idx_u,idx_c) );
	
end;

percent = ssr / size(log,1);
% if(nargout==3),
	model = model_data; 
% end;