% CUMULATE X BY ONE ACTION

% INPUT
% source_data - raw data structure from blendmodel_compute
% para - all model parameters
% OUTPUT
% ssr - sum of squared errors
% ssr_percent - % of sum of squared errors from total errors possible
% model - computed model

% WARNING combination of user_param & problem_param cannot exceed 2^WEIGHT_POWER


function model = cumulateX_each(source_data, model_data, log_idx, para)

% constants
IDX_USER = 1;
IDX_ACTIVITY = 2;
IDX_RESULT = 3;
IDX_DATE = 4;
% --

% params
problem_param = para(1);
problem_k = para(2);
practice_penalty = para(3);

if( problem_param < 0 || problem_k < 0 || practice_penalty < 0 ||...
		problem_param > 1 || problem_k > 1 || practice_penalty > 1),
	faul = 1; 
end;
% --

% ssr = 0;
% 
% n_users = size(source_data.user_id, 1);
% n_probl = size(source_data.problem_id, 1);
% n_concept = size(source_data.concept_id, 1);

% model_data = struct;
% model_data.model =				ones(  n_users, n_concept )*problem_k;
% model_data.solves =				zeros( n_users, n_probl   );
% model_data.con_lrnop =			zeros( n_users, n_concept ); 
% model_data.con_lrnop_corr =		zeros( n_users, n_concept ); 
% model_data.probl_lrnop =			zeros( n_users, n_probl   ); 
% model_data.probl_lrnop_corr =		zeros( n_users, n_probl   ); 
% model_data.pv_usr_log =			cell ( n_users, 1		  );
% for i=1:n_users, model_data.pv_usr_log{i} = []; end;
% 
% if(problem_id~=0),
% 	log_idx = find( source_data.problem_log(:,IDX_ACTIVITY)== problem_id );
% 	log = source_data.problem_log(log_idx,:);
% else
% 	log = source_data.problem_log;%(log_idx,:);
% end;

% for log_i = 1:size(log,1),
	
idx_u = source_data.problem_log(log_idx, IDX_USER);
idx_a = source_data.problem_log(log_idx, IDX_ACTIVITY);
res = source_data.problem_log(log_idx, IDX_RESULT);
idx_c = find( source_data.problem_concept(idx_a,:) )';
idx_c_dir = source_data.problem_concept_dir(idx_a,idx_c)';
idx_c_wght = source_data.problem_concept(idx_a,idx_c)';

out_w_sum = sum(idx_c_wght(find(idx_c_dir==1),:));
pre_w_sum = sum(idx_c_wght(find(idx_c_dir==0),:));

model_data.probl_lrnop     (idx_u,idx_a) = model_data.probl_lrnop     (idx_u,idx_a) + 1; % inc problem attempt
model_data.probl_lrnop_corr(idx_u,idx_a) = model_data.probl_lrnop_corr(idx_u,idx_a) + res; % inc problem attempt

model_data.con_lrnop     (idx_u,idx_c) = model_data.con_lrnop     (idx_u,idx_c) + 1;
model_data.con_lrnop_corr(idx_u,idx_c) = model_data.con_lrnop_corr(idx_u,idx_c) + res;

% compute ssr % pv log
% ssr = ssr + (res - model_data.solves(idx_u, idx_a))^2;
model_data.pv_usr_log{idx_u}  = [model_data.pv_usr_log{idx_u};  res, model_data.solves(idx_u, idx_a)];
model_data.pv_prob_log{idx_a} = [model_data.pv_prob_log{idx_a}; res, model_data.solves(idx_u, idx_a)];
% --

w_sum = ones(size(idx_c)) * pre_w_sum;
w_sum = w_sum .* (idx_c_dir~=1);
w_sum = w_sum + (idx_c_dir==1) * out_w_sum;

weight = idx_c_wght ./ w_sum;
prev_corr_attempts = model_data.probl_lrnop_corr(idx_u,idx_a) - res;
p_penalty = 1/((prev_corr_attempts+2)^practice_penalty);

if(isinf(p_penalty)),
	p_penalty = 1;
end;

% if not previously set, init
unset_idx = find(model_data.model(idx_u,idx_c)==0);
if( and( numel(unset_idx)>0, problem_k > 0 ) ),
	model_data.model(idx_u,idx_c(unset_idx)) = problem_k;
end;

model_data.model(idx_u,idx_c) = model_data.model(idx_u,idx_c) + ...
	res * (1-model_data.model(idx_u,idx_c)) .* weight' * problem_param * p_penalty;

model_data.solves(idx_u, idx_a) = prod( model_data.model(idx_u,idx_c) );
	
% end;

% percent = ssr / size(log,1);
model = model_data;