
% data array indexes
IDX_USER = 1;
IDX_ACTIVITY = 2;
IDX_RESULT = 3;
IDX_DATE = 4;
IDX_SUBACTIVITY = 5;

% 831; - no examples
groups = [404;535;592;759;760;850];

% load data
load('sql_SDATA.mat');
load('404_fitO_cum_3p_problem_all.mat');
load('404_fitO_cum_3p_problem_each.mat');
load('404_fit_cum_3p_user_all.mat');
load('404_fit_cum_3p_user_each.mat');
% load('404_fit_kt_4p_user_all.mat');
% load('404_fit_kt_4p_user_each.mat');
% load('404_fitO_kt_4p_problem_all.mat');
% load('404_fitO_kt_4p_problem_each.mat');

time = tic;

sql_models_context_fit_for404 = cell(size(groups,1),4);

%==================
for index=1:size(groups,1),
%==================
	source_data = SDATA{index};
	
	n_users = size(source_data.user_id, 1);
	n_probl = size(source_data.problem_id, 1);
	n_concept = size(source_data.concept_id, 1);
	
	model_data = struct;
	model_data.model =				zeros( n_users, n_concept );
	model_data.solves =				zeros( n_users, n_probl   );
	model_data.con_lrnop =			zeros( n_users, n_concept ); 
	model_data.con_lrnop_corr =		zeros( n_users, n_concept ); 
	model_data.probl_lrnop =		zeros( n_users, n_probl   ); 
	model_data.probl_lrnop_corr =	zeros( n_users, n_probl   ); 
	model_data.pv_usr_log =			cell ( n_users, 1		  );
	model_data.pv_prob_log =		cell ( n_probl, 1		  );
	for i=1:n_users, model_data.pv_usr_log{i}  = []; end;
	for i=1:n_probl, model_data.pv_prob_log{i} = []; end;

	% setup models
	cumL    = model_data;
	cumX_bg = model_data;
	cumX_al = model_data;
	cumX_al.model = cumX_al.model + params2odds(g404_all_cum_ko, g404_all_cum_user_param);
	cumX_ea = model_data;
	% Ko will be set automatically upon modeling
% 	kt_cl   = model_data;
% 	kt_cl.model = kt_cl.model + .5;
% 	kt_al   = model_data;
% 	kt_al.model = kt_al.model + params2odds(g404_all_kt_Lo, g404_all_kt_user_param);
% 	kt_ea   = model_data;
	
	for log_i = 1:size(source_data.problem_log,1),
		
		idx_u = source_data.problem_log(log_i, IDX_USER);
		idx_a = source_data.problem_log(log_i, IDX_ACTIVITY);
		res = source_data.problem_log(log_i, IDX_RESULT);
		idx_c = find( source_data.problem_concept(idx_a,:) )';
		
		cumL = cumulateL_each(source_data, cumL, log_i);
		cumX_bg = cumulateX_each(source_data, cumX_bg, log_i, [.5, 0, .25]);
		
		cumX_al = cumulateX_each(source_data, cumX_al, log_i, ...
			[params2odds(g404_all_cum_param, g404_all_cum_user_param),...
			 params2odds(g404_all_cum_ko,    g404_all_cum_user_param),...
			 params2odds(g404_all_cum_pp,    g404_all_cum_user_param)]);
		
		if( index == 1), % it is group 404 for which we do have USER PARAM
			cumX_ea = cumulateX_each(source_data, cumX_ea, log_i, ...
				[params2odds(g404_each_cum_pV(idx_a), g404_each_cum_user_param(idx_u)),...
				 params2odds(g404_each_cum_Ko(idx_a),    g404_each_cum_user_param(idx_u)),...
				 params2odds(g404_each_cum_oPP(idx_a),    g404_each_cum_user_param(idx_u))]);
		else % otherwise use global "all" USER PARAM
			cumX_ea = cumulateX_each(source_data, cumX_ea, log_i, ...
				[params2odds(g404_each_cum_pV(idx_a), g404_all_cum_user_param),...
				 params2odds(g404_each_cum_Ko(idx_a),    g404_all_cum_user_param),...
				 params2odds(g404_each_cum_oPP(idx_a),    g404_all_cum_user_param)]);
		end;
		
% 		kt_cl = knowtrace_each(source_data, kt_cl, log_i, [.5, .4, .2, .2]);
% 		kt_al = knowtrace_each(source_data, kt_al, log_i, ...
% 			[params2odds(g404_all_kt_Lo, g404_all_kt_user_param),...
% 			 params2odds(g404_all_kt_pT, g404_all_kt_user_param),...	
% 			 params2odds(g404_all_kt_pG, g404_all_kt_user_param),...
% 			 params2odds(g404_all_kt_pS, g404_all_kt_user_param)] );
% 		
% 		if( index == 1), % it is group 404 for which we do have USER PARAM
% 			kt_ea = knowtrace_each(source_data, kt_ea, log_i, ...
% 				[params2odds(g404_each_kt_Lo(idx_a), g404_each_kt_user_param(idx_u)),...
% 				 params2odds(g404_each_kt_pT(idx_a), g404_each_kt_user_param(idx_u)),...
% 				 params2odds(g404_each_kt_pG(idx_a), g404_each_kt_user_param(idx_u)),...
% 				 params2odds(g404_each_kt_pS(idx_a), g404_each_kt_user_param(idx_u))]);
% 		else % otherwise use global "all" USER PARAM
% 			kt_ea = knowtrace_each(source_data, kt_ea, log_i, ...
% 				[params2odds(g404_each_kt_Lo(idx_a), g404_all_kt_user_param),...
% 				 params2odds(g404_each_kt_pT(idx_a), g404_all_kt_user_param),...
% 				 params2odds(g404_each_kt_pG(idx_a), g404_all_kt_user_param),...
% 				 params2odds(g404_each_kt_pS(idx_a), g404_all_kt_user_param)]);
% 		end;
	end;
	
	sql_models_context_fit_for404{index,1} = cumL;
	sql_models_context_fit_for404{index,2} = cumX_bg;
	sql_models_context_fit_for404{index,3} = cumX_al;
	sql_models_context_fit_for404{index,4} = cumX_ea;
% 	sql_models_context_fit_for404{index,5} = kt_cl;
% 	sql_models_context_fit_for404{index,6} = kt_al;
% 	sql_models_context_fit_for404{index,7} = kt_ea;
%==================
end;
%==================

sql_models_context_fit_for404 = [...
	{'cumL','cumX_bg','cumX_al','cumX_ea'};...%,'kt_cl','kt_al','kt_ea'};...
	sql_models_context_fit_for404];

save('sql_models_context_fit_for404.mat','sql_models_context_fit_for404');
disp(['-- done in ',num2str(toc(time)),' --']);
beep;
