% THRUTHTABLE - turns actual/predicted binary classifier data into a (set
%		of) truth table(s) 
% data - 1st column actual binary output, 2nd..nth - modeled output
% thresholds - 1-m points of deciding whether modeled value is true
%		(>threshold) or false (<= threshold)
% result cell matrix of n-1.m 2x2 truthtables (col - actual, T,F, row -
%		modeled T, F) in a cell array
%
% Truth Table:
%						Actual
%                 Positive Negative
%                    .----|----.
%           Positive | TP | FP |
% Predicted          |----|----|
%           Negative | FN | TN |
%                    .----|----.
%

function truthtablecells = truthtable(data, threshold)

if(isempty(data)),
	truthtablecells = {[NaN NaN; NaN NaN]};
	return;
end;

tt_count = size(data,2)-1;
thres_count = numel(threshold);

truthtablecells = cell(tt_count, thres_count);

for i=1:tt_count,
	for j=1:thres_count,
		tf_colums = data( :, (2:tt_count+1) ) > threshold(j);

		truthtablecells{i,j} = zeros(2,2);
		truthtablecells{i,j}(1,1) = sum( (data(:,1)==1) .* (tf_colums(:,i)==1) ); % TP
		truthtablecells{i,j}(2,2) = sum( (data(:,1)==0) .* (tf_colums(:,i)==0) ); % TP
		truthtablecells{i,j}(1,2) = sum( (data(:,1)==0) .* (tf_colums(:,i)==1) ); % FP
		truthtablecells{i,j}(2,1) = sum( (data(:,1)==1) .* (tf_colums(:,i)==0) ); % FN
	end;
end;